import { LLMProvider } from '../../constants/llmProvider';
import { ILocalTool } from '../../types';
import { AgentStatus, IAgentTool, ISourceNode, SourceNodeType, IAgentInstructions, AgentAccessScope, AgentDelegationType } from '../../types/agents';
import { MemoryStrategy, MemoryType } from '../../types/memory';
import { Base } from '../base';
import { Configuration } from '../Configuration';
import { Tokens } from '../executions';
import { Execution } from '../executions/Execution';
import { Graph, GraphItem } from '../graphs';
import { KnowledgeBase } from '../knowledgeBases';
import { Memory, MemoryThread } from '../memory';
import { AgenticInterface, AgenticOperation } from '../tools';
import { ToolCall, ToolCallResult } from '../tools/ToolCall';
import { UserDetails } from '../UserDetails';
/**
 * Represents an agent in xpanderAI, managing tools, sessions, and operational workflows.
 * This class facilitates loading agents, handling tool executions, and managing prompt groups.
 */
export declare class Agent extends Base {
    configuration: Configuration;
    id: string;
    name: string;
    organizationId: string;
    status: AgentStatus;
    delegationType: AgentDelegationType;
    memoryType: MemoryType;
    memoryStrategy: MemoryStrategy;
    instructions: IAgentInstructions;
    accessScope: AgentAccessScope;
    sourceNodes: ISourceNode[];
    prompts: string[];
    tools: IAgentTool[];
    private _graph;
    knowledgeBases: KnowledgeBase[];
    oas: any;
    static getById(configuration: Configuration, agentId: string): Agent;
    /** Indicates if the agent is ready and tools are loaded. */
    ready: boolean;
    /** Collection of local tools specific to this agent. */
    localTools: ILocalTool[];
    execution?: Execution;
    userDetails?: UserDetails;
    executionMemory?: Memory;
    private shouldStop;
    private hitlIsRunning;
    private isLocalRun;
    private withAgentEndTool;
    graph: Graph;
    /** Maps original tool names to renamed versions for consistency. */
    protected originalToolNamesReMapping: Record<string, string>;
    /**
     * Constructs a new Agent instance.
     *
     * @param configuration - Configuration settings for the agent.
     * @param id - Unique identifier for the agent.
     * @param name - Human-readable name of the agent.
     * @param organizationId - Organization ID to which the agent belongs.
     * @param status - Current status of the agent.
     * @param delegationType - The agent's delegation type (Router/Sequence).
     * @param memoryType - Type of memory the agent utilizes.
     * @param memoryStrategy - Strategy for memory management.
     * @param instructions - Instructions for the agent's operation.
     * @param accessScope - Scope of the agent's access permissions.
     * @param sourceNodes - Source nodes associated with the agent.
     * @param prompts - Prompts used by the agent.
     * @param tools - Tools available to the agent.
     * @param graph - Graph structure representing the agent's operational flow.
     * @param knowledgeBases - Knowledge bases associated with the agent.
     */
    constructor(configuration: Configuration, id: string, name: string, organizationId: string, status: AgentStatus, delegationType: AgentDelegationType, memoryType: MemoryType, memoryStrategy: MemoryStrategy, instructions: IAgentInstructions, accessScope: AgentAccessScope, sourceNodes: ISourceNode[], prompts: string[], tools?: IAgentTool[], _graph?: any[], knowledgeBases?: KnowledgeBase[], oas?: any);
    /** Loads the agent data from its source node type. */
    load(agentId?: string, ignoreCache?: boolean, rawAgentData?: any): void;
    /**
     * Retrieves tools compatible with a specified LLM provider.
     *
     * @param llmProvider - The LLM provider to filter tools by (default: `OPEN_AI`).
     * @returns A list of tools matching the specified provider.
     */
    getTools(llmProvider?: LLMProvider): any[];
    /**
     * Adds messages to the memory thread.
     * Converts non-standard messages to a compatible format before storing.
     *
     * @param messages - An array of messages to be added to the memory thread.
     */
    addMessages(messages: any): void;
    /**
     * Adds tool call results as messages to the memory thread.
     *
     * @param toolCallResults - An array of tool call results to be added as messages.
     */
    addToolCallResults(toolCallResults: ToolCallResult[]): void;
    /**
     * Retrieves list of messages.
     *
     * @returns A list of messages according to the agent's llm provider.
     */
    get messages(): any[];
    /**
     * Gets the tool choice mode.
     * @returns {'required' | 'auto'} Returns 'required' if `withAgentEndTool` is true, otherwise 'auto'.
     */
    get toolChoice(): 'required' | 'auto';
    /** Constructs the API URL for this agent. */
    get url(): string;
    /**
     * Adds local tools to the agent with prefixed function names.
     *
     * @param tools - The list of local tools to add.
     */
    addLocalTools(tools: any[] | ILocalTool[]): void;
    /** Checks if the agent has local tools loaded. */
    get hasLocalTools(): boolean;
    private toolCallActionCounterKey;
    /**
     * Executes a single tool call and returns the result.
     *
     * @param tool - The tool call to execute.
     * @param payloadExtension - Additional payload data to merge.
     * @returns The result of the tool execution.
     */
    runTool(tool: ToolCall, payloadExtension?: any, isMultiple?: boolean): ToolCallResult;
    /**
     * Executes multiple tool calls sequentially and returns their results.
     *
     * @param toolCalls - The list of tool calls to execute.
     * @param payloadExtension - Additional payload data to merge.
     * @returns A list of results for each tool execution.
     */
    runTools(toolCalls: ToolCall[], payloadExtension?: any): ToolCallResult[];
    /** Retrieves the type of source node for the agent. */
    get sourceNodeType(): SourceNodeType;
    /**
     * Retrieves a node from the graph by its ID.
     *
     * @param itemId - The ID of the graph node to retrieve.
     * @returns The matching graph node, or undefined if not found.
     */
    retrieveNodeFromGraph(itemId: string): GraphItem | undefined;
    /**
     * Initializes the task execution for the agent.
     *
     * @param execution - The execution details.
     */
    initTask(execution: any): void;
    /**
     * Updates the user details for the agent.
     *
     * @param userDetails - The user details to update.
     */
    updateUserDetails(userDetails: UserDetails): void;
    /**
     * Retrieves the list of memory threads for the current user.
     *
     * @returns {MemoryThread[]} An array of memory threads associated with the user.
     * @throws {Error} If user details are missing, instructing to update user details first.
     */
    retrieveThreadsList(): MemoryThread[];
    /** Retrieves the vanilla knowledge bases of the agent. */
    get vanillaKnowledgeBases(): KnowledgeBase[];
    /** Retrieves the memory instance for the agent. */
    get memory(): Memory;
    /** Initializes the memory instance for the agent. */
    private initializeMemory;
    private get isAwaitingHITL();
    isFinished(): boolean;
    private switchExecution;
    addTask(input?: string, files?: string[], useWorker?: boolean, threadId?: string): Execution;
    stop(): void;
    disableAgentEndTool(): void;
    enableAgentEndTool(): void;
    retrieveExecutionResult(): Execution;
    /**
     * @function update
     * @description Updates the agent with the current instance's properties.
     * @returns {Agent} The updated agent's details.
     * @throws {Error} If the update process fails.
     * @memberof xpanderAI
     */
    update(): Agent;
    /**
     * @function sync
     * @description Syncs and deploys the agent along with its related assets.
     * @returns {Agent} The deployed agent's details.
     * @throws {Error} If the sync process fails.
     * @memberof xpanderAI
     */
    sync(): Agent;
    /**
     * Retrieves a list of available agentic interfaces.
     *
     * @param {boolean} [ignore_cache=false] - Whether to ignore cached data and fetch fresh data.
     * @returns {AgenticInterface[]} A list of agentic interfaces.
     * @throws {Error} If retrieval fails.
     *
     * @memberof xpander.ai
     */
    retrieveAgenticInterfaces(ignore_cache?: boolean): AgenticInterface[];
    /**
     * Retrieves a list of operations for a given agentic interface.
     *
     * @param {AgenticInterface} agenticInterface - The agentic interface to retrieve operations for.
     * @param {boolean} [ignore_cache=false] - Whether to ignore cached data and fetch fresh data.
     * @returns {AgenticOperation[]} A list of agentic operations.
     * @throws {Error} If retrieval fails.
     *
     * @memberof xpander.ai
     */
    retrieveAgenticOperations(agenticInterface: AgenticInterface, ignore_cache?: boolean): AgenticOperation[];
    /**
     * Attaches a list of agentic operations to the agent.
     *
     * @param {AgenticOperation[]} operations - The list of agentic operations to attach.
     * @throws {Error} If the attachment process fails.
     *
     * @memberof xpander.ai
     */
    attachOperations(operations: AgenticOperation[]): void;
    get endToolEnabled(): boolean;
    reportLlmUsage(llmResponse: any, llmInferenceDuration?: number, llmProvider?: LLMProvider, sourceNodeType?: string): void;
    reportExecutionMetrics(llmTokens: Tokens, aiModel?: string, sourceNodeType?: string): void;
}
